"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertAxiosResponseToCheckResult = exports.executeHttpRequest = void 0;
const log_1 = require("../../../log");
const version_1 = __importDefault(require("../../../version"));
const axios_1 = require("../../../axios");
const exponential_backoff_1 = require("../../retry/exponential-backoff");
async function executeHttpRequest(checkOptions, httpOptions) {
    log_1.log.debug({ checkId: checkOptions.id }, 'executing http check');
    try {
        const response = await (0, exponential_backoff_1.retry)(() => axios_1.axiosInstance.request({
            method: httpOptions.method,
            url: httpOptions.url,
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',
                'User-Agent': `broker client/${version_1.default} (http check service)`,
            },
            timeout: httpOptions.timeoutMs,
            validateStatus: () => true,
        }), { retries: 3, operation: `http check ${checkOptions.id}` });
        log_1.log.trace({ response: response.data }, 'http check raw response data');
        const checkResult = convertAxiosResponseToCheckResult({
            id: checkOptions.id,
            name: checkOptions.name,
            url: httpOptions.url,
            method: httpOptions.method,
        }, response);
        log_1.log.debug({ checkId: checkOptions.id, status: checkResult.status }, 'completed http check execution');
        return Promise.resolve(checkResult);
    }
    catch (error) {
        const errorMessage = `Error executing check with checkId ${checkOptions.id}`;
        log_1.log.debug({ error }, errorMessage);
        throw new Error(errorMessage);
    }
}
exports.executeHttpRequest = executeHttpRequest;
function convertAxiosResponseToCheckResult(check, response) {
    let status;
    if (response.status >= 200 && response.status <= 299) {
        status = 'passing';
    }
    else if (response.status == 429) {
        status = 'warning';
    }
    else {
        status = 'error';
    }
    const output = `HTTP ${check.method} ${check.url}: ${response.status} ${response.statusText}, response: ${truncate(JSON.stringify(response.data))}`;
    return {
        id: check.id,
        name: check.name,
        status,
        output,
    };
}
exports.convertAxiosResponseToCheckResult = convertAxiosResponseToCheckResult;
const truncate = (text) => {
    if (text.length <= 200) {
        return text;
    }
    const truncatedText = text.substring(0, 200);
    return `${truncatedText}... (truncated)`;
};
//# sourceMappingURL=http-executor.js.map