"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const primus_1 = __importDefault(require("primus"));
const request_promise_native_1 = __importDefault(require("request-promise-native"));
const uuid_1 = require("uuid");
const socket_1 = __importDefault(require("./socket"));
const log_1 = require("../log");
const relay_1 = require("../relay");
const webserver_1 = require("../webserver");
const version_1 = __importDefault(require("../version"));
const dispatcher_1 = require("./dispatcher");
const checks_1 = require("./checks");
const checks_handler_1 = require("./checks/api/checks-handler");
const scm_1 = require("./scm");
const main = async (clientOpts) => {
    var _a, _b, _c;
    log_1.log.info({ version: version_1.default }, 'running in client mode');
    if (!clientOpts.config.BROKER_CLIENT_URL) {
        const proto = !clientOpts.config.key && !clientOpts.config.cert ? 'http' : 'https';
        clientOpts.config.BROKER_CLIENT_URL = `${proto}://localhost:${clientOpts.port}`;
    }
    const brokerClientId = (0, uuid_1.v4)();
    log_1.log.info({ brokerClientId }, 'generated broker client id');
    let serverId;
    if ((0, dispatcher_1.highAvailabilityModeEnabled)(clientOpts.config)) {
        serverId = await (0, dispatcher_1.getServerId)(clientOpts.config, brokerClientId);
        if (serverId === null) {
            log_1.log.warn({}, 'could not receive server id from Broker Dispatcher');
            serverId = '';
        }
        else {
            log_1.log.info({ serverId }, 'received server id');
        }
    }
    let preflightCheckResults;
    if ((0, checks_1.preflightChecksEnabled)(clientOpts.config)) {
        // wrap preflight checks execution into try-catch, so the broker client
        // will start anyway
        try {
            preflightCheckResults = await (0, checks_1.executePreflightChecks)(clientOpts.config);
        }
        catch (error) {
            log_1.log.error({ error }, 'failed to execute preflight checks');
        }
    }
    if ((0, scm_1.commitSigningEnabled)(clientOpts.config)) {
        const commitSigningRules = (0, scm_1.commitSigningFilterRules)();
        (_a = clientOpts.filters.private) === null || _a === void 0 ? void 0 : _a.push(...commitSigningRules);
        log_1.log.info({ enabled: true, rulesCount: commitSigningRules.length }, 'loading commit signing rules');
    }
    const identifyingMetadata = {
        capabilities: ['post-streams'],
        clientId: brokerClientId,
        filters: clientOpts.filters,
        preflightChecks: preflightCheckResults,
        version: version_1.default,
    };
    const io = (0, socket_1.default)({
        token: clientOpts.config.brokerToken,
        url: clientOpts.config.brokerServerUrl,
        filters: (_b = clientOpts.filters) === null || _b === void 0 ? void 0 : _b.private,
        config: clientOpts.config,
        identifyingMetadata,
        serverId,
    });
    // start the local webserver to listen for relay requests
    const { app, server } = (0, webserver_1.webserver)(clientOpts.config, clientOpts.port);
    // IMPORTANT: defined before relay (`app.all('/*', ...`)
    app.get('/health/checks', (0, checks_handler_1.handleChecksRoute)(clientOpts.config));
    app.get('/health/checks/:checkId', (0, checks_handler_1.handleCheckIdsRoutes)(clientOpts.config));
    app.get(clientOpts.config.brokerHealthcheckPath || '/healthcheck', (req, res) => {
        // healthcheck state depends on websocket connection status
        // value of primus.Spark.OPEN means the websocket connection is open
        const isConnOpen = io.readyState === primus_1.default.Spark.OPEN;
        const status = isConnOpen ? 200 : 500;
        const data = {
            ok: isConnOpen,
            websocketConnectionOpen: isConnOpen,
            brokerServerUrl: io.url.href,
            version: version_1.default,
            transport: io.socket.transport.name,
        };
        return res.status(status).json(data);
    });
    app.get(clientOpts.config.brokerSystemcheckPath || '/systemcheck', async (req, res) => {
        // Systemcheck is the broker client's ability to assert the network
        // reachability and some correctness of credentials for the service
        // being proxied by the broker client.
        const brokerClientValidationMethod = clientOpts.config.brokerClientValidationMethod || 'GET';
        const brokerClientValidationTimeoutMs = clientOpts.config.brokerClientValidationTimeoutMs || 5000;
        const isJsonResponse = !clientOpts.config.brokerClientValidationJsonDisabled;
        // set auth header according to config
        const { auths, rawCreds } = loadCredentialsFromConfig(clientOpts.config);
        // make the internal validation request
        const validationResults = [];
        let errorOccurred = true;
        if (auths.length > 0) {
            for (let i = 0; i < auths.length; i++) {
                log_1.log.info(`Checking if credentials at index ${i} are valid`);
                const auth = auths[i];
                const rawCred = rawCreds[i];
                const { data, errorOccurred: err } = await checkCredentials(auth, clientOpts.config, brokerClientValidationMethod, brokerClientValidationTimeoutMs, isJsonResponse);
                data['maskedCredentials'] =
                    rawCred.length <= 6
                        ? '***'
                        : `${rawCred.substring(0, 3)}***${rawCred.substring(rawCred.length - 3)}`;
                validationResults.push(data);
                errorOccurred = err;
                log_1.log.info('Credentials checked');
            }
        }
        else {
            log_1.log.info('No credentials specified - checking if target can be accessed without credentials');
            const { data, errorOccurred: err } = await checkCredentials(null, clientOpts.config, brokerClientValidationMethod, brokerClientValidationTimeoutMs, isJsonResponse);
            data['maskedCredentials'] = null;
            validationResults.push(data);
            errorOccurred = err;
        }
        if (errorOccurred) {
            return res.status(500).json(validationResults);
        }
        else {
            return res.status(200).json(validationResults);
        }
    });
    // relay all other URL paths
    app.all('/*', (req, res, next) => {
        res.locals.io = io;
        next();
    }, (0, relay_1.forwardHttpRequest)((_c = clientOpts.filters) === null || _c === void 0 ? void 0 : _c.public));
    return {
        io,
        close: (done) => {
            log_1.log.info('client websocket is closing');
            server.close();
            io.destroy(function () {
                log_1.log.info('client websocket is closed');
                if (done) {
                    return done();
                }
            });
        },
    };
};
exports.main = main;
function credsFromHeader(s) {
    if (s.indexOf(' ') >= 0) {
        return s.substring(s.indexOf(' ') + 1);
    }
    else {
        return s;
    }
}
function loadCredentialsFromConfig(config) {
    let auths = [];
    let rawCreds = [];
    if (config.brokerClientValidationAuthorizationHeaderPool) {
        auths = config.brokerClientValidationAuthorizationHeaderPool;
        rawCreds =
            config.brokerClientValidationAuthorizationHeaderPool.map(credsFromHeader);
    }
    else if (config.brokerClientValidationBasicAuthPool) {
        auths = config.brokerClientValidationBasicAuthPool.map((s) => `Basic ${Buffer.from(s).toString('base64')}`);
        rawCreds = config.brokerClientValidationBasicAuthPool;
    }
    else if (config.brokerClientValidationAuthorizationHeader) {
        auths.push(config.brokerClientValidationAuthorizationHeader);
        rawCreds.push(config.brokerClientValidationAuthorizationHeader);
        rawCreds = rawCreds.map(credsFromHeader);
    }
    else if (config.brokerClientValidationBasicAuth) {
        auths.push(`Basic ${Buffer.from(config.brokerClientValidationBasicAuth).toString('base64')}`);
        rawCreds.push(config.brokerClientValidationBasicAuth);
    }
    return { auths, rawCreds };
}
async function checkCredentials(auth, config, brokerClientValidationMethod, brokerClientValidationTimeoutMs, isJsonResponse) {
    const data = {
        brokerClientValidationUrl: (0, log_1.sanitise)(config.brokerClientValidationUrl),
        brokerClientValidationMethod,
        brokerClientValidationTimeoutMs,
    };
    const validationRequestHeaders = {
        'user-agent': 'Snyk Broker client ' + version_1.default,
    };
    if (auth) {
        validationRequestHeaders['authorization'] = auth;
    }
    let errorOccurred = true;
    // This was originally `request`, but `await` is a lot easier to understand than nested callback hell.
    await (0, request_promise_native_1.default)({
        url: config.brokerClientValidationUrl,
        headers: validationRequestHeaders,
        method: brokerClientValidationMethod,
        timeout: brokerClientValidationTimeoutMs,
        json: isJsonResponse,
        resolveWithFullResponse: true,
        agentOptions: {
            ca: config.caCert, // Optional CA cert
        },
    })
        .then((response) => {
        // test logic requires to surface internal data
        // which is best not exposed in production
        if (process.env.JEST_WORKER_ID) {
            data['testResponse'] = response;
        }
        const responseStatusCode = response && response.statusCode;
        data['brokerClientValidationUrlStatusCode'] = responseStatusCode;
        // check for 2xx status code
        const goodStatusCode = /^2/.test(responseStatusCode);
        if (!goodStatusCode) {
            data['ok'] = false;
            data['error'] =
                responseStatusCode === 401 || responseStatusCode === 403
                    ? 'Failed due to invalid credentials'
                    : 'Status code is not 2xx';
            log_1.log.error(data, response && response.body, 'Systemcheck failed');
            return;
        }
        errorOccurred = false;
        data['ok'] = true;
    })
        .catch((error) => {
        // test logic requires to surface internal data
        // which is best not exposed in production
        if (process.env.JEST_WORKER_ID) {
            data['testError'] = error;
        }
        data['ok'] = false;
        data['error'] = error.message;
    });
    return { data, errorOccurred };
}
//# sourceMappingURL=index.js.map