"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.forwardWebSocketRequest = exports.forwardHttpRequest = exports.streamResponseHandler = exports.StreamResponseHandler = void 0;
const request_1 = __importDefault(require("request"));
const undefsafe_1 = __importDefault(require("undefsafe"));
const url_1 = require("url");
const url_2 = require("url");
const uuid_1 = require("uuid");
const filters_1 = __importDefault(require("./filters"));
const replace_vars_1 = require("./replace-vars");
const try_json_parse_1 = __importDefault(require("./try-json-parse"));
const log_1 = require("./log");
const version_1 = __importDefault(require("./version"));
const token_1 = require("./token");
const stream_1 = __importDefault(require("stream"));
const node_cache_1 = __importDefault(require("node-cache"));
const metrics_1 = require("./metrics");
const config_1 = require("./config");
const stream_posts_1 = require("./stream-posts");
const scm_1 = require("./client/scm");
let relayRequest = request_1.default;
relayRequest = request_1.default.defaults({
    ca: config_1.config.caCert,
    timeout: process.env.BROKER_DOWNSTREAM_TIMEOUT
        ? parseInt(process.env.BROKER_DOWNSTREAM_TIMEOUT)
        : 60000,
    agentOptions: {
        keepAlive: true,
        keepAliveMsecs: 60000,
        maxTotalSockets: 1000,
    },
});
class StreamResponseHandler {
    // streamBuffer;
    // response;
    // streamSize = 0;
    static create(streamingID) {
        const stream = streams.get(streamingID);
        if (!stream) {
            return null;
        }
        const streamResponse = stream;
        return new StreamResponseHandler(streamingID, streamResponse.streamBuffer, streamResponse.response);
    }
    constructor(streamingID, streamBuffer, response) {
        this.writeStatusAndHeaders = (statusAndHeaders) => {
            this.streamResponse.response
                .status(statusAndHeaders.status)
                .set(statusAndHeaders.headers);
        };
        this.writeChunk = (chunk, waitForDrainCb) => {
            this.streamResponse.streamSize += chunk.length;
            if (!this.streamResponse.streamBuffer.write(chunk) && waitForDrainCb) {
                waitForDrainCb(this.streamResponse.streamBuffer);
            }
        };
        this.finished = () => {
            this.streamResponse.streamBuffer.end();
            streams.del(this.streamingID);
            (0, metrics_1.observeResponseSize)({
                bytes: this.streamResponse.streamSize,
                isStreaming: true,
            });
        };
        this.destroy = (error) => {
            this.streamResponse.streamBuffer.destroy(error);
            streams.del(this.streamingID);
        };
        this.streamingID = streamingID;
        this.streamResponse = { streamBuffer, response, streamSize: 0 };
    }
}
exports.StreamResponseHandler = StreamResponseHandler;
const streams = new node_cache_1.default({
    stdTTL: parseInt(config_1.config.cacheExpiry) || 3600,
    checkperiod: parseInt(config_1.config.cacheCheckPeriod) || 60,
    useClones: false,
});
/**
 * @deprecated Deprecated in favour of {@link StreamResponseHandler} */
const streamResponseHandler = (token) => {
    return (streamingID, chunk, finished, ioResponse) => {
        const streamFromId = streams.get(streamingID);
        if (streamFromId) {
            const { streamBuffer, response } = streamFromId;
            let { streamSize } = streamFromId;
            if (streamBuffer) {
                if (ioResponse) {
                    response.status(ioResponse.status).set(ioResponse.headers);
                }
                if (chunk) {
                    streamSize += chunk.length;
                    streamBuffer.write(chunk);
                    streams.set(streamingID, { streamSize, streamBuffer, response });
                }
                if (finished) {
                    streamBuffer.end();
                    streams.del(streamingID);
                    (0, metrics_1.observeResponseSize)({
                        bytes: streamSize,
                        isStreaming: true,
                    });
                }
            }
            else {
                log_1.log.warn({ streamingID, token }, 'discarding binary chunk');
            }
        }
        else {
            log_1.log.warn({ streamingID, token }, 'trying to write into a closed stream');
        }
    };
};
exports.streamResponseHandler = streamResponseHandler;
// 1. Request coming in over HTTP conn (logged)
// 2. Filter for rule match (log and block if no match)
// 3. Relay over websocket conn (logged)
// 4. Get response over websocket conn (logged)
// 5. Send response over HTTP conn
const forwardHttpRequest = (filterRules) => {
    const filters = (0, filters_1.default)(filterRules);
    return (req, res) => {
        var _a;
        // If this is the server, we should receive a Snyk-Request-Id header from upstream
        // If this is the client, we will have to generate one
        (_a = req.headers)['snyk-request-id'] || (_a['snyk-request-id'] = (0, uuid_1.v4)());
        const logContext = {
            url: req.url,
            requestMethod: req.method,
            requestHeaders: req.headers,
            requestId: req.headers['snyk-request-id'] &&
                Array.isArray(req.headers['snyk-request-id'])
                ? req.headers['snyk-request-id'].join(',')
                : req.headers['snyk-request-id'] || '',
            maskedToken: req['maskedToken'],
            hashedToken: req['hashedToken'],
        };
        log_1.log.info(logContext, 'received request over HTTP connection');
        filters(req, (error, result) => {
            var _a, _b;
            if (error) {
                (0, metrics_1.incrementHttpRequestsTotal)(true);
                const reason = 'Request does not match any accept rule, blocking HTTP request';
                error.reason = reason;
                logContext.error = error;
                log_1.log.warn(logContext, reason);
                // TODO: respect request headers, block according to content-type
                return res
                    .status(401)
                    .send({ message: error.message, reason, url: req.url });
            }
            (0, metrics_1.incrementHttpRequestsTotal)(false);
            req.url = result.url;
            logContext.ioUrl = result.url;
            // check if this is a streaming request for binary data
            if (result.stream ||
                ((_b = (_a = res === null || res === void 0 ? void 0 : res.locals) === null || _a === void 0 ? void 0 : _a.capabilities) === null || _b === void 0 ? void 0 : _b.includes('post-streams'))) {
                const streamingID = (0, uuid_1.v4)();
                const streamBuffer = new stream_1.default.PassThrough({ highWaterMark: 1048576 });
                streamBuffer.on('error', (error) => {
                    // This may be a duplicate error, as the most likely cause of this is the POST handler calling destroy.
                    log_1.log.error({
                        ...logContext,
                        error,
                        stackTrace: new Error('stacktrace generator').stack,
                    }, 'caught error piping stream through to HTTP response');
                    res.destroy(error);
                });
                logContext.streamingID = streamingID;
                log_1.log.debug(logContext, 'sending stream request over websocket connection');
                streams.set(streamingID, {
                    response: res,
                    streamBuffer,
                    streamSize: 0,
                });
                streamBuffer.pipe(res);
                res.locals.io.send('request', {
                    url: req.url,
                    method: req.method,
                    body: req.body,
                    headers: req.headers,
                    streamingID,
                });
                return;
            }
            log_1.log.debug(logContext, 'sending request over websocket connection');
            // relay the http request over the websocket, handle websocket response
            res.locals.io.send('request', {
                url: req.url,
                method: req.method,
                body: req.body,
                headers: req.headers,
                streamingID: '',
            }, (ioResponse) => {
                logContext.responseStatus = ioResponse.status;
                logContext.responseHeaders = ioResponse.headers;
                logContext.responseBodyType = typeof ioResponse.body;
                const logMsg = 'sending response back to HTTP connection';
                if (ioResponse.status <= 200) {
                    log_1.log.debug(logContext, logMsg);
                    let responseBodyString = '';
                    if (typeof ioResponse.body === 'string') {
                        responseBodyString = ioResponse.body;
                    }
                    else if (typeof ioResponse.body === 'object') {
                        responseBodyString = JSON.stringify(ioResponse.body);
                    }
                    if (responseBodyString) {
                        const responseBodyBytes = Buffer.byteLength(responseBodyString, 'utf-8');
                        (0, metrics_1.observeResponseSize)({
                            bytes: responseBodyBytes,
                            isStreaming: false,
                        });
                    }
                    else {
                        // fallback metric to let us know if we're recording all response sizes
                        // we expect to remove this should it report 0
                        (0, metrics_1.incrementUnableToSizeResponse)();
                    }
                }
                else {
                    logContext.ioErrorType = ioResponse.errorType;
                    logContext.ioOriginalBodySize = ioResponse.originalBodySize;
                    log_1.log.warn(logContext, logMsg);
                }
                const httpResponse = res
                    .status(ioResponse.status)
                    .set(ioResponse.headers);
                const encodingType = (0, undefsafe_1.default)(ioResponse, 'headers.transfer-encoding');
                try {
                    // keep chunked http requests without content-length header
                    if (encodingType === 'chunked') {
                        httpResponse.write(ioResponse.body);
                        httpResponse.end();
                    }
                    else {
                        httpResponse.send(ioResponse.body);
                    }
                }
                catch (err) {
                    log_1.log.error({
                        ...logContext,
                        encodingType,
                        err,
                        stackTrace: new Error('stacktrace generator').stack,
                    }, 'error forwarding response from Web Socket to HTTP connection');
                }
            });
        });
    };
};
exports.forwardHttpRequest = forwardHttpRequest;
function legacyStreaming(logContext, rqst, config, io, streamingID) {
    let prevPartialChunk;
    let isResponseJson;
    log_1.log.warn(logContext, 'server did not advertise received-post-streams capability - falling back to legacy streaming');
    // Fall back to older streaming method if somehow connected to older server version
    rqst
        .on('response', (response) => {
        const status = (response && response.statusCode) || 500;
        logResponse(logContext, status, response, config);
        isResponseJson = isJson(response.headers);
        io.send('chunk', streamingID, '', false, {
            status,
            headers: response.headers,
        });
    })
        .on('data', (chunk) => {
        if (config.RES_BODY_URL_SUB && isResponseJson) {
            const { newChunk, partial } = (0, replace_vars_1.replaceUrlPartialChunk)(Buffer.from(chunk).toString(), prevPartialChunk, config);
            prevPartialChunk = partial;
            chunk = newChunk;
        }
        io.send('chunk', streamingID, chunk, false);
    })
        .on('end', () => {
        io.send('chunk', streamingID, '', true);
    })
        .on('error', (error) => {
        logError(logContext, error);
        io.send('chunk', streamingID, error.message, true, {
            status: 500,
        });
    });
}
// 1. Request coming in over websocket conn (logged)
// 2. Filter for rule match (log and block if no match)
// 3. Relay over HTTP conn (logged)
// 4. Get response over HTTP conn (logged)
// 5. Send response over websocket conn
const forwardWebSocketRequest = (filterRules, config, io, serverId) => {
    const filters = (0, filters_1.default)(filterRules);
    return (brokerToken) => (payload, emit) => {
        var _a, _b, _c;
        const requestId = payload.headers['snyk-request-id'];
        const logContext = {
            url: payload.url,
            requestMethod: payload.method,
            requestHeaders: payload.headers,
            requestId,
            streamingID: payload.streamingID,
            maskedToken: (0, token_1.maskToken)(brokerToken),
            hashedToken: (0, token_1.hashToken)(brokerToken),
            transport: (_c = (_b = (_a = io === null || io === void 0 ? void 0 : io.socket) === null || _a === void 0 ? void 0 : _a.transport) === null || _b === void 0 ? void 0 : _b.name) !== null && _c !== void 0 ? _c : 'unknown',
        };
        if (!requestId) {
            // This should be a warning but older clients won't send one
            // TODO make this a warning when significant majority of clients are on latest version
            log_1.log.trace(logContext, 'Header Snyk-Request-Id not included in headers passed through');
        }
        const realEmit = emit;
        emit = (responseData, isResponseFromRequestModule = false) => {
            var _a;
            // This only works client -> server, it's not possible to post data server -> client
            logContext.requestMethod = '';
            logContext.requestHeaders = {};
            if ((_a = io === null || io === void 0 ? void 0 : io.capabilities) === null || _a === void 0 ? void 0 : _a.includes('receive-post-streams')) {
                const postHandler = new stream_posts_1.BrokerServerPostResponseHandler(logContext, config, brokerToken, payload.streamingID, serverId, requestId);
                if (isResponseFromRequestModule) {
                    log_1.log.trace(logContext, 'posting streaming response back to Broker Server');
                    postHandler.forwardRequest(responseData);
                }
                else {
                    // Only for responses generated internally in the Broker Client/Server
                    postHandler.sendData(responseData);
                }
            }
            else {
                if (payload.streamingID) {
                    if (isResponseFromRequestModule) {
                        legacyStreaming(logContext, responseData, config, io, payload.streamingID);
                    }
                    else {
                        io.send('chunk', payload.streamingID, '', false, {
                            status: responseData.status,
                            headers: responseData.headers,
                        });
                        io.send('chunk', payload.streamingID, JSON.stringify(responseData.body), true);
                    }
                }
                else {
                    log_1.log.trace({ ...logContext, responseData }, 'sending fixed response over WebSocket connection');
                    realEmit(responseData);
                }
            }
        };
        log_1.log.info(logContext, 'received request over websocket connection');
        filters(payload, async (filterError, result) => {
            var _a;
            if (filterError) {
                (0, metrics_1.incrementWebSocketRequestsTotal)(true);
                const reason = 'Response does not match any accept rule, blocking websocket request';
                logContext.error = filterError;
                filterError.reason = reason;
                log_1.log.warn(logContext, reason);
                return emit({
                    status: 401,
                    body: {
                        message: filterError.message,
                        reason,
                        url: payload.url,
                    },
                });
            }
            (0, metrics_1.incrementWebSocketRequestsTotal)(false);
            if (result.url.startsWith('http') === false) {
                result.url = 'https://' + result.url;
                logContext.resultUrlSchemeAdded = true;
            }
            logContext.httpUrl = result.url;
            if (!payload.headers['user-agent']) {
                payload.headers['user-agent'] = 'Snyk Broker ' + version_1.default;
                logContext.userAgentHeaderSet = true;
            }
            if (result.auth) {
                payload.headers['authorization'] = result.auth;
                logContext.authHeaderSetByRuleAuth = true;
            }
            else {
                const parsed = (0, url_1.parse)(result.url);
                if (parsed.auth) {
                    // if URL contains basic auth,
                    // remove authorization header to prefer auth on the URL.
                    if (parsed.auth.includes(':')) {
                        delete payload.headers['authorization'];
                    }
                    // if URL contains token auth,
                    // put the token in the authorization header
                    // instead of on the URL.
                    else {
                        payload.headers['authorization'] = `token ${parsed.auth}`;
                        // then strip from the url
                        parsed.auth = null;
                        result.url = (0, url_2.format)(parsed);
                    }
                    logContext.authHeaderSetByRuleUrl = true;
                }
            }
            // if the request is all good - and at this point it is, we'll check
            // whether we want to do variable substitution on the body
            //
            // Variable substitution - for those who forgot - is substituting a part
            // of a given string (e.g. "${SOME_ENV_VAR}/rest/of/string")
            // with an env var of the same name (SOME_ENV_VAR).
            // This is used (for example) to substitute the snyk url
            // with the broker's url when defining the target for an incoming webhook.
            if (!config.disableBodyVarsSubstitution && payload.body) {
                const parsedBody = (0, try_json_parse_1.default)(payload.body);
                if (parsedBody.BROKER_VAR_SUB) {
                    logContext.bodyVarsSubstitution = parsedBody.BROKER_VAR_SUB;
                    for (const path of parsedBody.BROKER_VAR_SUB) {
                        let source = (0, undefsafe_1.default)(parsedBody, path); // get the value
                        source = (0, replace_vars_1.replace)(source, config); // replace the variables
                        (0, undefsafe_1.default)(parsedBody, path, source); // put it back in
                    }
                    payload.body = JSON.stringify(parsedBody);
                }
            }
            if (!config.disableHeaderVarsSubstitution &&
                payload.headers &&
                payload.headers['x-broker-var-sub']) {
                // check whether we want to do variable substitution on the headers
                logContext.headerVarsSubstitution = payload.headers['x-broker-var-sub'];
                for (const path of payload.headers['x-broker-var-sub'].split(',')) {
                    let source = (0, undefsafe_1.default)(payload.headers, path.trim()); // get the value
                    source = (0, replace_vars_1.replace)(source, config); // replace the variables
                    (0, undefsafe_1.default)(payload.headers, path.trim(), source); // put it back in
                }
            }
            // remove headers that we don't want to relay
            // (because they corrupt the request)
            [
                'x-forwarded-for',
                'x-forwarded-proto',
                'content-length',
                'host',
                'accept-encoding',
                'content-encoding',
            ].map((_) => delete payload.headers[_]);
            if (config.removeXForwardedHeaders === 'true') {
                for (const key in payload.headers) {
                    if (key.startsWith('x-forwarded-')) {
                        delete payload.headers[key];
                    }
                }
                if (payload.headers['forwarded']) {
                    delete payload.headers['forwarded'];
                }
            }
            if (brokerToken && (io === null || io === void 0 ? void 0 : io.socketType) === 'server') {
                Object.assign(payload.headers, { 'X-Broker-Token': brokerToken });
            }
            log_1.log.debug(logContext, 'sending websocket request over HTTP connection');
            // Sometimes we receive the body as a {type, data} object
            // Unsure why - possibly Primus?
            if (((_a = payload.body) === null || _a === void 0 ? void 0 : _a.type) === 'Buffer')
                payload.body = Buffer.of(payload.body.data);
            // Request library is buggy and will throw an error if we're POST'ing an empty body without an explicit Content-Length header
            if (!payload.body || payload.body.length === 0) {
                payload.headers['Content-Length'] = '0';
            }
            payload.headers['connection'] = 'Keep-Alive';
            payload.headers['Keep-Alive'] = 'timeout=60, max=1000';
            if ((0, scm_1.gitHubTreeCheckNeeded)(config, {
                method: payload.method,
                url: payload.url,
            })) {
                try {
                    (0, scm_1.validateGitHubTreePayload)(payload.body);
                }
                catch (error) {
                    log_1.log.error({ error }, 'error while checking github tree payload for symlinks');
                    return emit({
                        status: 401,
                        body: error.message,
                    });
                }
            }
            if ((0, scm_1.gitHubCommitSigningEnabled)(config, {
                method: payload.method,
                url: payload.url,
            })) {
                try {
                    payload.body = await (0, scm_1.signGitHubCommit)(config, payload.body);
                }
                catch (error) {
                    log_1.log.error({ error }, 'error while signing github commit');
                }
            }
            const req = {
                url: result.url,
                headers: payload.headers,
                method: payload.method,
                body: payload.body,
            };
            // check if this is a streaming request for binary data
            if (payload.streamingID) {
                log_1.log.debug(logContext, 'serving stream request');
                try {
                    emit(relayRequest(req), true);
                }
                catch (e) {
                    log_1.log.error({
                        ...logContext,
                        error: e,
                        stackTrace: new Error('stacktrace generator').stack,
                    }, 'caught error sending HTTP response over WebSocket');
                }
                return;
            }
            relayRequest(req, (error, response, responseBody) => {
                if (error) {
                    logError(logContext, error);
                    return emit({
                        status: 500,
                        body: error.message,
                    });
                }
                const contentLength = responseBody.length;
                // Note that the other side of the request will also check the length and will also reject it if it's too large
                // Set to 20MB even though the server is 21MB because the server looks at the total data travelling through the websocket,
                // not just the size of the body, so allow 1MB for miscellaneous data (e.g., headers, Primus overhead)
                const maxLength = parseInt(config.socketMaxResponseLength) || 20971520;
                if (contentLength && contentLength > maxLength) {
                    const errorMessage = `body size of ${contentLength} is greater than max allowed of ${maxLength} bytes`;
                    logError(logContext, {
                        errorMessage,
                    });
                    return emit({
                        status: 502,
                        errorType: 'BODY_TOO_LARGE',
                        originalBodySize: contentLength,
                        body: {
                            message: errorMessage,
                        },
                    });
                }
                const status = (response && response.statusCode) || 500;
                if (config.RES_BODY_URL_SUB && isJson(response.headers)) {
                    const replaced = (0, replace_vars_1.replaceUrlPartialChunk)(responseBody, null, config);
                    responseBody = replaced.newChunk;
                }
                logResponse(logContext, status, response, config);
                emit({ status, body: responseBody, headers: response.headers });
            });
        });
    };
};
exports.forwardWebSocketRequest = forwardWebSocketRequest;
function isJson(responseHeaders) {
    return responseHeaders['content-type']
        ? responseHeaders['content-type'].includes('json')
        : false;
}
function logResponse(logContext, status, response, config) {
    logContext.responseStatus = status;
    logContext.responseHeaders = response.headers;
    logContext.responseBody =
        config && config.LOG_ENABLE_BODY === 'true' ? response.body : null;
    log_1.log.info(logContext, 'sending response back to websocket connection');
}
function logError(logContext, error) {
    log_1.log.error({
        ...logContext,
        error,
        stackTrace: new Error('stacktrace generator').stack,
    }, 'error while sending websocket request over HTTP connection');
}
//# sourceMappingURL=relay.js.map