"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const log_1 = require("../log");
const socket_1 = __importDefault(require("./socket"));
const relay_1 = require("../relay");
const version_1 = __importDefault(require("../version"));
const token_1 = require("../token");
const metrics_1 = require("../metrics");
const prometheus_middleware_1 = require("./prometheus-middleware");
const broker_middleware_1 = require("./broker-middleware");
const webserver_1 = require("../webserver");
const dispatcher_1 = require("../dispatcher");
const main = async (serverOpts) => {
    var _a, _b;
    log_1.log.info({ version: version_1.default }, 'running in server mode');
    // start the local webserver to listen for relay requests
    const { app, server } = (0, webserver_1.webserver)(serverOpts.config, serverOpts.port);
    // Requires are done recursively, so this is here to avoid contaminating the Client
    const onSignal = async () => {
        log_1.log.debug('received exit signal, closing server');
        await (0, dispatcher_1.serverStopping)(() => {
            process.exit(0);
        });
    };
    process.once('SIGINT', onSignal);
    process.once('SIGTERM', onSignal);
    await (0, dispatcher_1.serverStarting)();
    // bind the socket server to the web server
    const { io, connections } = (0, socket_1.default)({
        server,
        filters: (_a = serverOpts.filters) === null || _a === void 0 ? void 0 : _a.private,
        config: serverOpts.config,
    });
    if (!process.env.JEST_WORKER_ID) {
        app.use((0, prometheus_middleware_1.applyPrometheusMiddleware)());
    }
    app.get('/connection-status/:token', (req, res) => {
        const token = req.params.token;
        const maskedToken = (0, token_1.maskToken)(token);
        const hashedToken = (0, token_1.hashToken)(token);
        if (connections.has(token)) {
            const clientsMetadata = connections.get(token).map((conn) => ({
                version: conn.metadata && conn.metadata.version,
                filters: conn.metadata && conn.metadata.filters,
            }));
            return res.status(200).json({ ok: true, clients: clientsMetadata });
        }
        log_1.log.warn({ maskedToken, hashedToken }, 'no matching connection found');
        return res.status(404).json({ ok: false });
    });
    app.all('/broker/:token/*', (req, res, next) => {
        const token = req.params.token;
        const maskedToken = (0, token_1.maskToken)(token);
        const hashedToken = (0, token_1.hashToken)(token);
        req['maskedToken'] = maskedToken;
        req['hashedToken'] = hashedToken;
        // check if we have this broker in the connections
        if (!connections.has(token)) {
            (0, metrics_1.incrementHttpRequestsTotal)(false);
            log_1.log.warn({ maskedToken, hashedToken }, 'no matching connection found');
            return res.status(404).json({ ok: false });
        }
        // Grab a first (newest) client from the pool
        // This is really silly...
        res.locals.io = connections.get(token)[0].socket;
        res.locals.socketVersion = connections.get(token)[0].socketVersion;
        res.locals.capabilities = connections.get(token)[0].metadata.capabilities;
        req['locals'] = {};
        req['locals']['capabilities'] =
            connections.get(token)[0].metadata.capabilities;
        // strip the leading url
        req.url = req.url.slice(`/broker/${token}`.length);
        log_1.log.debug({ url: req.url }, 'request');
        next();
    }, broker_middleware_1.validateBrokerTypeMiddleware, (0, relay_1.forwardHttpRequest)((_b = serverOpts.filters) === null || _b === void 0 ? void 0 : _b.public));
    app.post('/response-data/:brokerToken/:streamingId', (req, res) => {
        (0, metrics_1.incrementHttpRequestsTotal)(false);
        const token = req.params.brokerToken;
        const streamingID = req.params.streamingId;
        const maskedToken = (0, token_1.maskToken)(token);
        const hashedToken = (0, token_1.hashToken)(token);
        const logContext = {
            hashedToken,
            maskedToken,
            streamingID,
            requestId: req.headers['snyk-request-id'],
        };
        log_1.log.info(logContext, 'Handling response-data request');
        req['maskedToken'] = maskedToken;
        req['hashedToken'] = hashedToken;
        const streamHandler = relay_1.StreamResponseHandler.create(streamingID);
        if (!streamHandler) {
            log_1.log.error(logContext, 'unable to find request matching streaming id');
            res
                .status(500)
                .json({ message: 'unable to find request matching streaming id' });
            return;
        }
        let statusAndHeaders = '';
        let statusAndHeadersSize = -1;
        req
            .on('data', function (data) {
            try {
                log_1.log.trace({ ...logContext, dataLength: data.length }, 'Received data event');
                let bytesRead = 0;
                if (statusAndHeadersSize === -1) {
                    bytesRead += 4;
                    statusAndHeadersSize = data.readUInt32LE();
                    log_1.log.debug({ ...logContext, statusAndHeadersSize }, 'request metadata size read from stream');
                }
                if (statusAndHeadersSize > 0 &&
                    statusAndHeaders.length < statusAndHeadersSize) {
                    const endPosition = Math.min(bytesRead + statusAndHeadersSize - statusAndHeaders.length, data.length);
                    log_1.log.trace({ ...logContext, bytesRead, endPosition }, 'Reading ioJson');
                    statusAndHeaders += data.toString('utf8', bytesRead, endPosition);
                    bytesRead = endPosition;
                    if (statusAndHeaders.length === statusAndHeadersSize) {
                        log_1.log.trace({ ...logContext, statusAndHeaders }, 'Converting to json');
                        const statusAndHeadersJson = JSON.parse(statusAndHeaders);
                        const logData = {
                            ...logContext,
                            responseStatus: statusAndHeadersJson.status,
                            responseHeaders: statusAndHeadersJson.headers,
                        };
                        const logMessage = 'Handling response-data request - io bits';
                        if (statusAndHeadersJson.status > 299 &&
                            statusAndHeadersJson.status !== 404) {
                            log_1.log.info(logData, logMessage);
                        }
                        else {
                            log_1.log.debug(logData, logMessage);
                        }
                        streamHandler.writeStatusAndHeaders(statusAndHeadersJson);
                    }
                    else {
                        log_1.log.trace({
                            ...logContext,
                            currentSize: statusAndHeaders.length,
                            expectedSize: statusAndHeadersSize,
                        }, 'Was unable to fit all information into a single data object');
                    }
                }
                if (bytesRead < data.length) {
                    log_1.log.trace(logContext, 'Handling response-data request - data part');
                    streamHandler.writeChunk(data.subarray(bytesRead, data.length), (streamBuffer) => {
                        log_1.log.trace(logContext, 'pausing request stream');
                        req.pause();
                        streamBuffer.once('drain', () => {
                            log_1.log.trace(logContext, 'resuming request stream');
                            req.resume();
                        });
                    });
                }
            }
            catch (e) {
                log_1.log.error({ ...logContext, statusAndHeaders, statusAndHeadersSize, error: e }, 'caught error handling data event for streaming HTTP response');
            }
        })
            .on('end', function () {
            log_1.log.debug(logContext, 'Handling response-data request - end part');
            streamHandler.finished();
            res.status(200).json({});
        })
            .on('error', (err) => {
            log_1.log.error({ ...logContext, error: err }, 'received error handling POST from client');
            streamHandler.destroy(err);
            res.status(500).json({ err });
        });
    });
    app.get('/', (req, res) => res.status(200).json({ ok: true, version: version_1.default }));
    app.get('/healthcheck', (req, res) => res.status(200).json({ ok: true, version: version_1.default }));
    return {
        io,
        close: (done) => {
            log_1.log.info('server websocket is closing');
            server.close();
            io.destroy(function () {
                log_1.log.info('server websocket is closed');
                if (done) {
                    return done();
                }
            });
        },
    };
};
exports.main = main;
//# sourceMappingURL=index.js.map