"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const primus_1 = __importDefault(require("primus"));
const primus_emitter_1 = __importDefault(require("primus-emitter"));
const log_1 = require("../log");
const relay_1 = require("../relay");
const token_1 = require("../token");
const metrics_1 = require("../metrics");
const dispatcher_1 = require("../dispatcher");
exports.default = ({ server, filters, config }) => {
    // Requires are done recursively, so this is here to avoid contaminating the Client
    const ioConfig = {
        transformer: 'engine.io',
        parser: 'EJSON',
        maxLength: parseInt(config.socketMaxResponseLength) || 22020096,
        transport: {
            allowEIO3: true,
            pingInterval: parseInt(config.socketPingInterval) || 25000,
            pingTimeout: parseInt(config.socketPingTimeout) || 20000,
        },
        compression: Boolean(config.socketUseCompression) || false,
    };
    const io = new primus_1.default(server, ioConfig);
    io.socketType = 'server';
    io.socketVersion = 1;
    io.plugin('emitter', primus_emitter_1.default);
    log_1.log.info(ioConfig, 'using io config');
    const connections = new Map();
    const response = (0, relay_1.forwardWebSocketRequest)(filters, config, io);
    const streamingResponse = relay_1.streamResponseHandler;
    io.on('error', (error) => log_1.log.error({ error }, 'Primus/engine.io server error'));
    io.on('connection', function (socket) {
        let token = socket.request.uri.pathname
            .replaceAll(/\/primus\/([^/]+)\//g, '$1')
            .toLowerCase();
        let clientId = null;
        let clientVersion = null;
        let identified = false;
        log_1.log.info({ maskedToken: (0, token_1.maskToken)(token), hashedToken: (0, token_1.hashToken)(token) }, 'new client connection');
        socket.send('identify', { capabilities: ['receive-post-streams'] });
        const close = (closeReason = 'none') => {
            if (token) {
                const maskedToken = (0, token_1.maskToken)(token);
                const hashedToken = (0, token_1.hashToken)(token);
                if (identified) {
                    const clientPool = connections
                        .get(token)
                        .filter((_) => _.socket !== socket);
                    log_1.log.info({
                        closeReason,
                        maskedToken,
                        hashedToken,
                        remainingConnectionsCount: clientPool.length,
                    }, 'client connection closed');
                    if (clientPool.length) {
                        connections.set(token, clientPool);
                    }
                    else {
                        log_1.log.info({ maskedToken, hashedToken }, 'removing client');
                        connections.delete(token);
                    }
                    (0, metrics_1.decrementSocketConnectionGauge)();
                }
                else {
                    log_1.log.warn({ maskedToken, hashedToken }, 'client disconnected before identifying itself');
                }
                setImmediate(async () => await (0, dispatcher_1.clientDisconnected)(token, clientId));
            }
        };
        // TODO decide if the socket doesn't identify itself within X period,
        // should we toss it away?
        socket.on('identify', (clientData) => {
            // clientData can be a string token coming from older broker clients,
            // OR an object coming from newer clients in the form of { token, metadata }
            if (typeof clientData === 'object') {
                token = clientData.token && clientData.token.toLowerCase();
            }
            else {
                token = clientData.toLowerCase(); // lowercase to standardise tokens
                // stub a proper clientData, signal client is too old
                clientData = { token, metadata: { version: 'pre-4.27' } };
            }
            if (!token) {
                log_1.log.warn({ token, metadata: metadataWithoutFilters(clientData.metadata) }, 'new client connection identified without a token');
                return;
            }
            const maskedToken = (0, token_1.maskToken)(token);
            const hashedToken = (0, token_1.hashToken)(token);
            log_1.log.info({
                maskedToken,
                hashedToken,
                metadata: metadataWithoutFilters(clientData.metadata),
            }, 'new client connection identified');
            const clientPool = connections.get(token) || [];
            clientPool.unshift({
                socket,
                socketType: 'server',
                socketVersion: 1,
                metadata: clientData.metadata,
            });
            connections.set(token, clientPool);
            socket.on('chunk', streamingResponse(token));
            socket.on('request', response(token));
            socket.on('incoming::ping', (time) => {
                setImmediate(async () => await (0, dispatcher_1.clientPinged)(token, clientId, clientVersion, time));
            });
            clientId = clientData.metadata.clientId;
            clientVersion = clientData.metadata.version;
            setImmediate(async () => await (0, dispatcher_1.clientConnected)(token, clientId, clientVersion));
            (0, metrics_1.incrementSocketConnectionGauge)();
            identified = true;
        });
        ['close', 'end', 'disconnect'].forEach((e) => socket.on(e, () => close(e)));
        socket.on('error', (error) => {
            log_1.log.warn({ error }, 'error on websocket connection');
        });
    });
    return { io, connections };
};
const metadataWithoutFilters = (metadataWithFilters) => {
    return {
        capabilities: metadataWithFilters.capabilities,
        clientId: metadataWithFilters.clientId,
        preflightChecks: metadataWithFilters.preflightChecks,
        version: metadataWithFilters.version,
    };
};
//# sourceMappingURL=socket.js.map