"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.webserver = void 0;
const express_1 = __importDefault(require("express"));
const body_parser_1 = __importDefault(require("body-parser"));
const fs_1 = __importDefault(require("fs"));
const log_1 = require("./log");
const token_1 = require("./token");
const http_1 = require("http");
const https_1 = require("https");
// bodyparser < 2 initializes req.body to {} for requests with no body. This
// breaks later serialization, so this pair of middlewares ensures that requests
// with no body have req.body = undefined. This matches the as-yet-unreleased
// bodyparser 2.x behaviour.
const EmptyBody = Symbol('Empty Body');
const markEmptyRequestBody = (req, res, next) => {
    req.body = req.body || EmptyBody;
    next();
};
const stripEmptyRequestBody = (req, res, next) => {
    if (req.body === EmptyBody) {
        delete req.body;
    }
    next();
};
const extractBrokerTokenFromUrl = (urlString) => {
    const regex = /^\/broker\/([a-z0-9-]+)\//;
    return urlString.match(regex) ? urlString.match(regex)[1] : null;
};
const webserver = (config, altPort) => {
    let { httpsKey = null, httpsCert = null, port = 7341 } = config; // eslint-disable-line prefer-const
    const isHttp = !httpsKey && !httpsCert; // no https if there's no certs
    const app = (0, express_1.default)();
    app.disable('x-powered-by');
    app.use(markEmptyRequestBody);
    app.use(body_parser_1.default.raw({
        type: (req) => req.headers['content-type'] !==
            'application/vnd.broker.stream+octet-stream',
        limit: '10mb',
    }));
    app.use(stripEmptyRequestBody);
    app.use((err, req, res, next) => {
        if (err) {
            const brokerToken = extractBrokerTokenFromUrl(req.url);
            const maskedToken = (0, token_1.maskToken)(brokerToken);
            const hashedToken = (0, token_1.hashToken)(brokerToken);
            log_1.log.error({
                url: req.url.replaceAll(brokerToken, maskedToken),
                requestMethod: req.method,
                requestHeaders: req.headers,
                requestId: req.headers['snyk-request-id'] || '',
                maskedToken: maskedToken,
                hashedToken: hashedToken,
                error: err,
            }, 'Error occurred receiving http request on broker webserver');
            next(err);
        }
        else {
            next(err);
        }
    });
    if (altPort) {
        port = altPort;
    }
    log_1.log.info({ port }, 'local server is listening');
    const server = isHttp
        ? (0, http_1.createServer)(app).listen(port)
        : (0, https_1.createServer)({
            key: fs_1.default.readFileSync(httpsKey),
            cert: fs_1.default.readFileSync(httpsCert),
        }, app).listen(port);
    return { app, server };
};
exports.webserver = webserver;
//# sourceMappingURL=webserver.js.map